-------------------------------------------------------------------------------
-- Title       : Parallel-prefix incrementer
-- Project     : VHDL Library of Arithmetic Units
-------------------------------------------------------------------------------
-- File        : Inc.vhd
-- Author      : Reto Zimmermann  <zimmi@iis.ee.ethz.ch>
-- Company     : Integrated Systems Laboratory, ETH Zurich
-- Date        : 1997/11/04
-------------------------------------------------------------------------------
-- Copyright (c) 1998 Integrated Systems Laboratory, ETH Zurich
-------------------------------------------------------------------------------
-- Description :
-- Incrementer using parallel-prefix propagate-lookahead logic.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
library synergy;  use synergy.signed_arith.all;
library arith_lib;
use arith_lib.arith_lib.all;

-------------------------------------------------------------------------------

entity Inc is

  generic (width : positive := 8;  	-- word width
	   speed : speedType := fast);  -- performance parameter

  port (A : in std_logic_vector(width-1 downto 0);  -- operand
	Z : out std_logic_vector(width-1 downto 0));  -- result

end Inc;

-------------------------------------------------------------------------------

architecture Behavioral of Inc is

  signal Auns, Zuns : unsigned(width-1 downto 0);  -- unsigned

begin

  -- type conversion: std_logic_vector -> unsigned
  Auns <= unsigned(A);

  -- increment
  Zuns <= Auns + 1;

  -- type conversion: unsigned -> std_logic_vector
  Z <= std_logic_vector(Zuns);

end Behavioral;

-------------------------------------------------------------------------------

architecture Structural of Inc is 

  signal PO : std_logic_vector(width-1 downto 0);  -- prefix propagate out

begin

  -- calculate prefix output propagate signal
  prefix : PrefixAnd
    generic map (width, speed)
    port map (A, PO);

  -- calculate result bits
  Z <= A xor PO(width-2 downto 0) & '1';

end Structural;

-------------------------------------------------------------------------------
